###############################################################################
#                                                                              
# Rohde & Schwarz GmbH & Co. KG                                                
# 1MA294 Sigfox Signal Generation and Analysis                                 
# Testing Sigfox Devices with FSx and VSE/VSE-K70                              
# Version 1.2, Andreas Mandel (Dept. 1MAA/RSD)                                 
#                                                                              
###############################################################################

import visa
import time

# Variables #

port  = "COM4"      # Serial Port                                              
inst  = "FPL"       # Instrument Alias                                         

ul_frq = 868150000  # Uplink Freq. in Hz                                       
dl_frq = 869525000  # Downlink Freq. in Hz                                     

srat_1 = 240E3      # Sample Rate in Hz (Capturing IQ)                         
swpt   = 8          # Measurement Time in s                                    
tlev   = -20        # Trigger Level in dBm                                     
leng   = 524288     # Points determined by FTT Calculation                     

srat_2 = 100        # Sample Rate in Hz (VSA)                                  
rlen   = 250        # Record Length in Sym                                     
dsym   = 115        # Number of displayed Symbols                              
tmap   = -6         # Trigger Level in dBm                                     

# Target File for I/Q Data #

file   = "C:\\ProgramData\\Rohde-Schwarz\\VSE\\IQ-Data\\Sigfox_Data.iq.tar"

# User Information #

print("----------------------------------------------------------------------")
print("\nRohde & Schwarz GmbH & Co. KG")
print("1MA294 - Sigfox Signal Generation and Analysis")
print("Testing Sigfox Devices with FSx and VSE/VSE-K70\n")
print("----------------------------------------------------------------------")

# Connect Module via COM Port

rm = visa.ResourceManager()

ti = rm.open_resource(port)
ti.timeout = None
ti.baud_rate = 9600
ti.data_bits = 8
ti.read_termination = '\r'
ti.parity = visa.constants.Parity.none
ti.stop_bits = visa.constants.StopBits.one

print("----------------------------------------------------------------------")
print("Sigfox Module Settings.")
print("----------------------------------------------------------------------")

## Get Device ID                                                               
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @rspnd: <dev_id><CR>                                                        
id = ti.query("AT$ID?")
print "Device ID: " + str(id.replace("\n", ""))

## Get Uplink Frequency                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <ul_freq><CR>                                                       
ul = ti.query("AT$IF?")
print "Uplink Frequency: " + str(ul.replace("\n", "") + " Hz")

## Get Downlink Frequency                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <dl_freq><CR>                                                       
dl = ti.query("AT$DR?")
print "Downlink Frequency: " + str(dl.replace("\n", "") + " Hz")

print("----------------------------------------------------------------------")
print("Setting new Module Configuration...")
print("----------------------------------------------------------------------")

## Set Uplink Frequency                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <ul_freq>: Central Uplink Frequency (Hz)                            
## @rspnd:  OK<CR>                                                             
print "Setting Uplink Frequency..."
stat = ti.query("AT$IF=" + str(ul_frq))
print "State: " + str(stat.replace("\n", ""))

## Set Downlink Frequency                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <dl_freq>: Central Downlink Frequency (Hz)                          
## @rspnd:  OK<CR>                                                             
print "Setting Downlink Frequency..."
stat = ti.query("AT$DR=" + str(dl_frq))
print "State: " + str(stat.replace("\n", ""))

# R&S VSE: Capturing I/Q Data #

print("----------------------------------------------------------------------")
print("R&S VSE: Capturing I/Q Data")
print("----------------------------------------------------------------------")

VSE.write("DEV:REC '" + str(inst) + "'")                                        # Connect to R&S Analyzer.
VSE.ask("*OPC?")                                                                # Wait for VSE Explorer...

print("")

VSE.write("*RST")
VSE.write("*CLS")
VSE.ask("*OPT?")
VSE.ask("*IDN?")

print("")

VSE.write("INST:BLOCK:CHAN:DEV '" + str(inst) + "'")                            # Specified <INSTR> as Input Source.                    
VSE.write("FREQ:CENT " + str(ul_frq))                                           # Define Center Frequency.                              
VSE.write("TRAC:IQ:SRAT " + str(srat_1))                                        # Define Sample Rate.                                   
VSE.write("IQ:BAND:MODE FFT")                                                   # Set RBW determined by FFT Parameters.                 
VSE.write("IQ:FFT:ALG AVER")                                                    # Set FFT Calculation Method to Average.                
VSE.write("IQ:FFT:LENG " + str(leng))                                           # Define Number of Points determined by FFT Calculation.
VSE.write("SWE:TIME " + str(swpt))                                              # Define Measurement Time.                              
VSE.write("IQ:FFT:ALG SING")                                                    # ...                                                   
VSE.write("IQ:FFT:ALG AVER")                                                    # Workaround for R&S VSE 1.40 Beta!                     
VSE.write("SWE:TIME " + str(swpt))                                              # ...
VSE.write("TRIG:SOUR IFP")                                                      # Select IF Power Trigger Source.                       
VSE.write("TRIG:LEV:IFP " + str(tlev))                                          # Define Power Level for Trigger.                       
VSE.ask("*OPC?")                                                                # Wait for VSE Explorer...                              

print("")

VSE.write("INIT:CONT ON")                                                       # Switche ON continuous Measurement.                    
VSE.write("INST:BLOC:CHAN:REC")                                                 # Start capturing I/Q Data to temporary File...         

time.sleep(2.5)                                                                 # Synchronization!                                      

# Sigfox Module: Sending TX Packages #

print("----------------------------------------------------------------------")
print("Sigfox Module: Sending TX Packages.")
print("----------------------------------------------------------------------")

## Send Bit Status                                                             
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @param: <stat_val>: Status Value 1/0                                        
## @rspnd:  OK<CR>                                                             
print "Sending Bit Status..."
stat = ti.query("AT$SB=1")
print "State: " + str(stat.replace("\n", ""))

# Waiting for VSE... #

print("----------------------------------------------------------------------")
print("Waiting for R&S VSE...")
print("----------------------------------------------------------------------")

VSE.ask("*OPC?")

print("----------------------------------------------------------------------")

# R&S VSE: Performing Vector Signal Analysis #

print("----------------------------------------------------------------------")
print("R&S VSE: Performing Vector Signal Analysis.")
print("----------------------------------------------------------------------")

VSE.write("EXP:IQ:FORM IQTAR")                                                  # Store Data to an iq.tar File.        
VSE.write("EXP:IQ:FILE '" + str(file) + "'")                                    # Write captured I/Q Data to File.     

freq = [ ]
VSE.write("CALC2:MARK1 ON")                                                     # Turn ON Marker for Frame 1.          
VSE.write("CALC2:MARK1:X?")                                                     # Query Transmit Frequency for Frame 1.
freq.append(VSE.read())                                                         # Read Value for Frame 1.              
VSE.write("CALC2:DELT1 ON")                                                     # Turn ON Marker for Frame 2.          
VSE.write("CALC2:DELT1:X?")                                                     # Query Transmit Frequency for Frame 2.
freq.append(VSE.read())                                                         # Read Value for Frame 2.              
VSE.write("CALC2:DELT2 ON")                                                     # Turn ON Marker for Frame 3.          
VSE.write("CALC2:DELT2:X?")                                                     # Query Transmit Frequency for Frame 3.
freq.append(VSE.read())                                                         # Read Value for Frame 3.              

for i in range(0, 3):

    print("")
    
    VSE.write("INST:CRE DDEM, 'VSA " + str(i+1) + "'")                          # Create VSA Channel.                   
    VSE.write("INST:BLOC:CHAN:FILE '" + str(file) + "'")                        # Assign iq.tar File as Input Source.    
    VSE.write("SENS:DDEM:FORM PSK")                                             # Set PSK for Demodulation Mode.        
    VSE.write("SENS:DDEM:PSK:FORM NORM")                                        # Define Demodulation Order for PSK.    
    VSE.write("SENS:DDEM:TFIL OFF")                                             # Switch OFF Transmit Filter.           
    VSE.write("SENS:DDEM:SRAT " + str(srat_2) + "Hz")                           # Define Symbol Rate.                   
    VSE.write("SENS:DDEM:RLEN " + str(rlen) + "Sym")                            # Define Capture Length.                
    VSE.write("SENS:DDEM:TIME " + str(dsym) + "Sym")                            # Determine Number of displayed Symbols.
    VSE.write("FREQ:CENT " + str(freq[i]) + "Hz")                               # Define Center Frequency.              
    VSE.write("TRIG:SEQ:SOUR MAGN")                                             # Select Magnitude Trigger Source.      
    VSE.write("TRIG:LEV:MAP " + str(tmap) + "dBm")                              # Define Power Level for Trigger.       
    VSE.ask("*OPC?")                                                            # Wait for VSE Explorer...              

    VSE.write("INIT:CONT OFF")                                                  # Switch Measurement Mode to Single.    
    VSE.write("INIT;*WAI")                                                      # Start Measuring...                    

    VSE.write("SENS:DDEM:SEAR:MBUR:CALC 1");                                    # Set Result Range.                     

print("----------------------------------------------------------------------")

# Disconnect Module #

ti.close()
rm.close()

# User Information #

print("----------------------------------------------------------------------")
print("Executing Script finished!")
print("----------------------------------------------------------------------")