###############################################################################
#
# Rohde & Schwarz GmbH & Co. KG
# 1MA294 SigFox Signal Generation and Analysis
# Frequency and Timing for Sigfox Devices
# Version 1.0, Andreas Mandel (Dept. 1MAA/RSD)
#
###############################################################################

import visa
import time
import math

# Variables #

port   = "COM4"    # COM Port                                                  
ul_frq = 868130000 # Uplink Frequency in Hz                                    
dl_frq = 869525000 # Downlink Frequency in Hz                                  

spn    = 400e3     # Frequency Span in Hz                                      
rbw    = 100       # Resolution Bandwidth in Hz                                
vbw    = 100       # Video Bandwidth in Hz                                     
att    = 10        # Reference Level Offset in dB                              
off    = 0         # FPL Input Attenuation in dB                               
rlev   = 10        # Reference Level in dBm                                    

pack_count = 3     # Packet Counter                                            


# User Information #

print("----------------------------------------------------------------------")
print("\nRohde & Schwarz GmbH & Co. KG")
print("1MA294 - SigFox Signal Generation and Analysis")
print("Frequency and Timing for Sigfox Devices\n")
print("----------------------------------------------------------------------")

# R&S FPL Configuration #

print("----------------------------------------------------------------------")
print("R&S FPL Configuration.")
print("----------------------------------------------------------------------")
    
FPL.write("*RST")                                                               # Reset Device.                             
FPL.write("*CLS")                                                               # Clear Status.                             
FPL.ask("*OPT?")                                                                # Option Query.                             
FPL.ask("*IDN?")                                                                # Query ID String.                          

print("")

FPL.write("SYST:DISP:UPD ON")                                                   # Display Update ON during Remote Control.  
FPL.write("INST:SEL SANALYZER")                                                 # Activate Spectrum Analyzer Mode.          
FPL.write("FREQ:CENT " + str(ul_frq))                                           # Define Center Frequency.                  
FPL.write("FREQ:SPAN " + str(spn))                                              # Define Frequency Span.                    

FPL.write("SENS:DET APE")                                                       # Define Trace Detector.                    
FPL.write("SENS:BAND:RES " + str(rbw))                                          # Set Resolution Bandwidth.                 
FPL.write("SENS:BAND:VID " + str(vbw))                                          # Set Video Bandwidth.                      
FPL.write("INP:ATT:AUTO ON")                                                    # Define Input Attenuation.                 
FPL.write("DISP:TRAC:Y:RLEV " + str(rlev))                                      # Set Reference Level.                      
FPL.write("DISP:TRAC:Y:RLEV:OFFS " + str(off))                                  # Set Reference Level Offset.               

FPL.write("DISP:TRAC:Y:SPAC LOG")                                               # Define logarithmic Scaling for Y-Axis.    
FPL.write("DISP:TRAC:Y 100dB")                                                  # Set Display Range to 100 dB.              
FPL.write("CALC:SGR ON")                                                        # Switch ON Spectrogram Display.            
FPL.ask("*OPC?")                                                                # Wait for Analyzer...                      

print("")

# Connect Module via COM Port #

rm = visa.ResourceManager()

ti = rm.open_resource(port)
ti.timeout = None
ti.baud_rate = 9600
ti.data_bits = 8
ti.read_termination = '\r'
ti.parity = visa.constants.Parity.none
ti.stop_bits = visa.constants.StopBits.one

print("----------------------------------------------------------------------")
print("Current Module Settings.")
print("----------------------------------------------------------------------")

###############################################################################
## Read Device ID.                                                             
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @rspnd: <dev_id><CR>                                                        
id = ti.query("AT$ID?")
print "Device ID: " + str(id.replace("\n", ""))
###############################################################################


###############################################################################
## Get Sigfox Version.                                                         
## @ref:    SIGFOX Protocol V1 Library API Documentation                       
##          Literature Number: N/A                                             
## @rspnd: <api><CR>                                                           
## @rspnd: <lib><CR>                                                           
ti.write("AT$V?")
api = ti.read()
print "Sigfox API: " + api.replace("\n", "").upper()
ti.read_termination = '\r\n'
lib = ti.read()
print "Sigfox Library: " + lib.replace("\n","").upper()
ti.read_termination = '\r'
###############################################################################

print("")

###############################################################################
## Get Uplink Frequency.                                                       
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <ul_freq><CR>                                                       
ul = ti.query("AT$IF?")
print "Uplink Frequency: " + str(ul.replace("\n", "") + " Hz")
###############################################################################


###############################################################################
## Get Downlink Frequency.                                                     
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <dl_freq><CR>                                                       
dl = ti.query("AT$DR?")
print "Downlink Frequency: " + str(dl.replace("\n", "") + " Hz")
###############################################################################

print("")

print("----------------------------------------------------------------------")
print("Setting new Module Configuration.")
print("----------------------------------------------------------------------")

###############################################################################
## Set Uplink Frequency.                                                       
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <ul_freq>: Central Uplink Frequency (Hz)                            
## @rspnd:  OK<CR>                                                             
print "Setting Uplink Frequency..."
stat = ti.query("AT$IF=" + str(ul_frq))
print "State: " + str(stat.replace("\n", "") + "\n")
###############################################################################


###############################################################################
## Set Downlink Frequency.                                                     
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <dl_freq>: Central Downlink Frequency (Hz)                          
## @rspnd:  OK<CR>                                                             
print "Setting Downlink Frequency..."
stat = ti.query("AT$DR=" + str(dl_frq))
print "State: " + str(stat.replace("\n", "") + "\n")
###############################################################################

print("----------------------------------------------------------------------")
print("Sending SigFox Uplink Frames.")
print("----------------------------------------------------------------------")

###############################################################################
## Send Frame in Uplink                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @param: <ul_data>: 0 to 12 Bytes of Uplink Message                          
## @rspnd:  OK<CR>                                                             
for i in range(0, pack_count):  
    print("Package Nr. " + str(i + 1) + "...")
    stat = ti.query("AT$SF=1")
    print("State: " + str(stat.replace("\n", "") + "\n"))
    time.sleep(3.5)
###############################################################################

print("----------------------------------------------------------------------")

# Disconnect Module #

ti.close()
rm.close()

# User Information #

print("----------------------------------------------------------------------")
print("Executing Script finished!")
print("----------------------------------------------------------------------")