###############################################################################
#                                                                              
# Rohde & Schwarz GmbH & Co. KG                                                
# 1MA294 Sigfox Signal Generation and Analysis                                 
# Production Testing with Sigfox Devices                                       
# Version 1.0, Andreas Mandel (Dept. 1MAA/RSD)                                 
#                                                                              
###############################################################################

import visa
import math

# Variables #

port    = "COM4"    # COM Port                                                 

ul_frq  = 868130000 # Uplink Freq. in Hz                                       
dl_frq  = 869525000 # Downlink Freq. in Hz                                     

att_in  =   0.00    # External Attenuation (Input) in dB                       
enp     =  16.00    # Expected Nominal Power in dBm                            
fsp     =   1.25    # Frequency Span in MHz                                    
fftl    =   1024    # FFT Length (Number of Samples)                           
thr     = -30.00    # Trigger Threshold in dBm                                 
tout    =  20.00    # Trigger Timeout in dBm                                   

att_out =   0.00    # External Attenuation (Output) in dB                      
lev     = -30.00    # Level (RMS) in dBm                                       
dga     =   0.00    # Digital Gain in dB                                       

# Arbitrary Waveform File #

arbfile = "C:\ProgramData\Rohde-Schwarz\CMW\Data\Waveform\KV687_600bps_2GFSK_SIGFOX_Waveform.wv"

# User Information #

print("----------------------------------------------------------------------")
print("\nRohde & Schwarz GmbH & Co. KG")
print("1MA294 - Sigfox Signal Generation and Analysis")
print("Production Testing with Sigfox Devices\n")
print("----------------------------------------------------------------------")

# R&S FSV Configuration #

print("----------------------------------------------------------------------")
print("R&S CMW100: General Purpose RF Measurement.")
print("----------------------------------------------------------------------")
    
CMW.write("*RST")                                                               # Reset Device                           
CMW.write("*CLS")                                                               # Clear Status                           
CMW.ask("*OPT?")                                                                # Option Query                           
CMW.ask("*IDN?")                                                                # Query ID String                        

CMW.write("SYST:DISP:UPD ON")

CMW.write("ROUT:GPRF:MEAS:SCEN:SAL R11, RX11")

CMW.write("CONF:GPRF:MEAS:RFS:EATT " + str(att_in))
CMW.write("CONF:GPRF:MEAS:RFS:FREQ " + str(ul_frq))
CMW.write("CONF:GPRF:MEAS:RFS:ENP  " + str(enp))

CMW.write("TRIG:GPRF:MEAS:FFTS:SOUR \"IF Power\"")
CMW.write("TRIG:GPRF:MEAS:FFTS:SLOP REDG")
CMW.write("TRIG:GPRF:MEAS:FFTS:THR " + str(thr))
CMW.write("TRIG:GPRF:MEAS:FFTS:TOUT " + str(tout))

CMW.write("CONF:GPRF:MEAS:FFTS:REP CONT")
CMW.write("CONF:GPRF:MEAS:FFTS:FSP " + str(fsp) + " MHz")
CMW.write("CONF:GPRF:MEAS:FFTS:FFTL " + str(fftl))
CMW.write("CONF:GPRF:MEAS:FFTS:DET PEAK")
CMW.write("CONF:GPRF:MEAS:FFTS:AMOD LOG")

#CMW.write("CONF:GPRF:MEAS:FFTS:PSE:NOAM 5")
#CMW.write("CONF:GPRF:MEAS:FFTS:PSE OFF, -100E+01, +100E1, OFF, -300E+01, -100E-01, OFF, +100E+01, +300E+01, OFF, -500E+01, -300E+01, OFF, +300E+01, +500E+01")

CMW.write("INIT:GPRF:MEAS:FFTS")

print("----------------------------------------------------------------------")

# Connect Module via COM Port

rm = visa.ResourceManager()

ti = rm.open_resource(port)
ti.timeout = None
ti.baud_rate = 9600
ti.data_bits = 8
ti.read_termination = '\r'
ti.parity = visa.constants.Parity.none
ti.stop_bits = visa.constants.StopBits.one

print("----------------------------------------------------------------------")
print("Sigfox Module: Settings.")
print("----------------------------------------------------------------------")

## Get Device ID                                                               
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @rspnd: <dev_id><CR>                                                        
id = ti.query("AT$ID?")
print "Device ID: " + str(id.replace("\n", ""))

## Get Uplink Frequency                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <ul_freq><CR>                                                       
ul = ti.query("AT$IF?")
print "Uplink Frequency (Hz): " + str(ul.replace("\n", ""))

## Get Downlink Frequency                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <dl_freq><CR>                                                       
dl = ti.query("AT$DR?")
print "Downlink Frequency (Hz): " + str(dl.replace("\n", ""))

print("----------------------------------------------------------------------")
print("Writing new Module Configuration...")
print("----------------------------------------------------------------------")

## Set Uplink Frequency                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <ul_freq>: Central Uplink Frequency (Hz)                            
## @rspnd:  OK<CR>                                                             
print "Setting Uplink Frequency..."
stat = ti.query("AT$IF=" + str(ul_frq))
print "State: " + str(stat.replace("\n", ""))

## Set Downlink Frequency                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <dl_freq>: Central Downlink Frequency (Hz)                          
## @rspnd:  OK<CR>                                                             
print "Setting Downlink Frequency..."
stat = ti.query("AT$DR=" + str(dl_frq))
print "State: " + str(stat.replace("\n", ""))

## Send Status in Uplink.                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @param: <stat_val>: Status Value 1/0                                        
## @rspnd:  OK<CR>                                                             
#print "Sending Status in Uplink..."                                           
#stat = ti.query("AT$SB=1")                                                    
#print "State: " + str(stat.replace("\n", "") + "\n")                          
## Send Status in Uplink and requests Downlink Message (ACK).                  
#print "Sending Status in Uplink and request ACK..."                           
#stat = ti.query("AT$SB=1,1")                                                  

## Send Frame in Uplink                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @param: <ul_data>: 0 to 12 Bytes of Uplink Message                          
## @rspnd:  OK<CR>                                                             
#print "Sending Frame in Uplink..."
#stat = ti.query("AT$SF=123ABC")
#print "State: " + str(stat.replace("\n", "") + "\n")
## Send Frame in Uplink and request Downlink Message (ACK)                     
#print "Sending Frame in Uplink and request ACK..."
#stat = ti.query("AT$SF=123ABC,1")

## Configure Device in Test Mode                                               
## @ref:    SIGFOX Protocol V1 Library API Documentation                       
##          Literature Number: N/A                                             
## @param: <count>: Test Mode...                                               
##                  SFX_TEST_MODE_TX_BPSK     = 0,                             
##                  SFX_TEST_MODE_TX_PROTOCOL = 1,                             
##                  SFX_TEST_MODE_RX_PROTOCOL = 2,                             
##                  SFX_TEST_MODE_RX_GFSK     = 3,                             
##                  SFX_TEST_MODE_RX_SENSI    = 4,                             
##                  SFX_TEST_MODE_TX_SYNTH    = 5                              
## @param: <confg>: Configuration Parameter, depends on Test Mode.             
## @rspnd:  OK<CR>                                                             
print("----------------------------------------------------------------------")
print("Running Module in TX_BPSK Test Mode...")
ti.write("AT$TM=0," + str(1))
print("----------------------------------------------------------------------")
print("State: OK")
print("----------------------------------------------------------------------")

#plib.reporting.off()
#print("----------------------------------------------------------------------")
#print("Finishing Module...")
#state = ti.read()
#print("----------------------------------------------------------------------")
#print "State: " + str(stat.replace("\n", ""))
#print("----------------------------------------------------------------------")
#plib.reporting.on()

# Disconnect Module #

ti.close()
rm.close()

# User Information #

print("----------------------------------------------------------------------")
print("Executing Script finished!")
print("----------------------------------------------------------------------")