###############################################################################
#                                                                              
# Rohde & Schwarz GmbH & Co. KG                                                
# 1MA294 Sigfox Signal Generation and Analysis                                 
# Spectrum Analysis for Sigfox Devices                                         
# Version 1.3, Andreas Mandel (Dept. 1MAA/RSD)                                 
#                                                                              
###############################################################################

import visa
import math

# Variables #

port   = "COM4"    # COM Port                                                  

ul_frq = 868130000 # Uplink Frequency in Hz                                    
dl_frq = 869525000 # Downlink Frequency in Hz                                  

spn    = 2E+3      # Frequency Span in Hz                                      
rbw    = 10        # Resolution Bandwidth in Hz                                
vid    = 10        # Video Bandwidth in Hz                                     
swpt   = 3         # Measurement Time in s                                     
att    = 10        # Reference Level Offset in dB                              
off    = 0         # Input Attenuation in dB                                   
rlev   = 10        # Reference Level in dBm                                    
swe    = 2.25      # Sweep Time in s                                           
ifp    = -65       # IF Power in dBm                                           

pack_count = 9     # Number of expected Packets                                
trac_count = 3     # Traces per Measurement Channel                            

rslt = ""          # Result String                                             

## Limits for Spectrum - ETSI                                                  
## @ref:    Ultra Narrow Band Transceiver Product Requirements Specification   
##          Literature Number: PRS-UNVT_RF, August 2015, PDF, p. 10            

shif = 1.0E-5      # Shifts all Y-Values of the Limit Line by ... dB           

xlw = [ -960E2, -500E0, -300E0, -100, 100E0, 300E0, 500E0 ] # LowerValue in  Hz
xup = [ -500E0, -300E0, -100E0, +100, 300E0, 500E0, 960E2 ] # UpperValue in  Hz
lim = [    -50,    -40,    -20,    0,   -20,   -40,   -50 ] # LimitValue in dBc

# User Information #

print("----------------------------------------------------------------------")
print("\nRohde & Schwarz GmbH & Co. KG")
print("1MA294 - Sigfox Signal Generation and Analysis")
print("Spectrum Analysis for Sigfox Devices\n")
print("----------------------------------------------------------------------")

# R&S FPL Configuration #

print("----------------------------------------------------------------------")
print("R&S Analyzer: Configuration.")
print("----------------------------------------------------------------------")
    
FPL.write("*RST")                                                               # Reset Device.                           
FPL.write("*CLS")                                                               # Clear Status.                           
FPL.ask("*OPT?")                                                                # Option Query.                           
FPL.ask("*IDN?")                                                                # Query ID String.                        

print("")

FPL.write("SYST:DISP:UPD ON")                                                   # Display Update ON during Remote Control.
FPL.write("INST:SEL SANALYZER")                                                 # Activate Spectrum Analyzer Mode.        
FPL.write("INP:ATT:AUTO ON")                                                    # Set Input Attenuation.                  
FPL.write("FREQ:CENT " + str(ul_frq))                                           # Define Center Frequency.                
FPL.write("FREQ:SPAN " + str(spn))                                              # Define Frequency Span.                  
FPL.write("SENS:BAND " + str(rbw))                                              # Define Resolution Bandwidth.            
FPL.write("SENS:BAND:VID " + str(vid))                                          # Define Video Bandwidth.                 
FPL.write("SENS:SWE:TIME " + str(swe))                                          # Define Duration of Single Sweep.        
FPL.write("DISP:TRAC:Y:RLEV " + str(rlev))                                      # Define Reference Level.                 
FPL.write("DISP:TRAC:Y:RLEV:OFFS " + str(off))                                  # Define Reference Level Offset.          
FPL.write("TRIG:SOUR IFP")                                                      # Select IF Power Trigger Source.         
FPL.write("TRIG:LEV:IFP " + str(ifp))                                           # Define Power Level for Trigger.         

# Connect Module via COM Port #

rm = visa.ResourceManager()

ti = rm.open_resource(port)
ti.timeout = None
ti.baud_rate = 9600
ti.data_bits = 8
ti.read_termination = '\r'
ti.parity = visa.constants.Parity.none
ti.stop_bits = visa.constants.StopBits.one

print("----------------------------------------------------------------------")
print("Sigfox Module: Settings.")
print("----------------------------------------------------------------------")

## Get Device ID                                                               
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @rspnd: <dev_id><CR>                                                        
id = ti.query("AT$ID?")
print "Device ID: " + str(id.replace("\n", ""))

## Get Uplink Frequency                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <ul_freq><CR>                                                       
ul = ti.query("AT$IF?")
print "Uplink Frequency (Hz): " + str(ul.replace("\n", ""))

## Get Downlink Frequency                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @rspnd: <dl_freq><CR>                                                       
dl = ti.query("AT$DR?")
print "Downlink Frequency (Hz): " + str(dl.replace("\n", ""))

print("----------------------------------------------------------------------")
print("Writing new Module Configuration...")
print("----------------------------------------------------------------------")

## Set Uplink Frequency                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <ul_freq>: Central Uplink Frequency (Hz)                            
## @rspnd:  OK<CR>                                                             
print "Setting Uplink Frequency..."
stat = ti.query("AT$IF=" + str(ul_frq))
print "State: " + str(stat.replace("\n", ""))

## Set Downlink Frequency                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 15                  
## @param: <dl_freq>: Central Downlink Frequency (Hz)                          
## @rspnd:  OK<CR>                                                             
print "Setting Downlink Frequency..."
stat = ti.query("AT$DR=" + str(dl_frq))
print "State: " + str(stat.replace("\n", ""))

## Send Status in Uplink.                                                      
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @param: <stat_val>: Status Value 1/0                                        
## @rspnd:  OK<CR>                                                             
#print "Sending Status in Uplink..."                                           
#stat = ti.query("AT$SB=1")                                                    
#print "State: " + str(stat.replace("\n", "") + "\n")                          
## Send Status in Uplink and requests Downlink Message (ACK).                  
#print "Sending Status in Uplink and request ACK..."                           
#stat = ti.query("AT$SB=1,1")                                                  

## Send Frame in Uplink                                                        
## @ref:    CC112x TI-SIGFOX SDK Demo - Getting Started Guide                  
##          Literature Number: SWRU487, June 2016, PDF, p. 14                  
## @param: <ul_data>: 0 to 12 Bytes of Uplink Message                          
## @rspnd:  OK<CR>                                                             
#print "Sending Frame in Uplink..."                                            
#stat = ti.query("AT$SF=123ABC")                                               
#print "State: " + str(stat.replace("\n", "") + "\n")                          
## Send Frame in Uplink and request Downlink Message (ACK)                     
#print "Sending Frame in Uplink and request ACK..."                            
#stat = ti.query("AT$SF=123ABC,1")                                             

## Configure Device in Test Mode                                               
## @ref:    SIGFOX Protocol V1 Library API Documentation                       
##          Literature Number: N/A                                             
## @param: <count>: Test Mode...                                               
##                  SFX_TEST_MODE_TX_BPSK     = 0,                             
##                  SFX_TEST_MODE_TX_PROTOCOL = 1,                             
##                  SFX_TEST_MODE_RX_PROTOCOL = 2,                             
##                  SFX_TEST_MODE_RX_GFSK     = 3,                             
##                  SFX_TEST_MODE_RX_SENSI    = 4,                             
##                  SFX_TEST_MODE_TX_SYNTH    = 5                              
## @param: <confg>: Configuration Parameter, depends on Test Mode.             
## @rspnd:  OK<CR>                                                             
print("----------------------------------------------------------------------")
print("Running Module in TX_BPSK Test Mode...")
ti.write("AT$TM=0," + str(pack_count))
print("----------------------------------------------------------------------")
print("State: OK")
print("----------------------------------------------------------------------")

print("----------------------------------------------------------------------")
print("R&S Analyzer: Spectrum Measurement.")
print("----------------------------------------------------------------------")

n = 0
freq = []
powr = []

for i in range(1, pack_count + 1):
    
    n = n + 1
   
    FPL.write("DISP:TRAC" + str(n) + " ON")                                     # Switch ON new Trace.                 
    FPL.write("DISP:TRAC" + str(n) + ":MODE MAXH")                              # Set Trace Mode to max. Hold.         

    FPL.write("INIT" + str(n) + ":CONT OFF")                                    # Select Single Sweep Mode.            
    FPL.write("INIT" + str(n))                                                  # Initiate Measurement.                

    FPL.write("*WAI")                                                           # Wait for Anazlyer...                 

    FPL.write("CALC:MARK ON")                                                   # Turn ON Marker.                      
    FPL.write("CALC:MARK:TRAC " + str(n))                                       # Set Marker to current Trace.         
    FPL.write("CALC:MARK:MAX")                                                  # Move Marker to Carrier.              

    FPL.write("CALC:MARK:X?")                                                   # Query Carrier Frequency.             
    freq.append(float(FPL.read()))                                              # Read Value...                        
    FPL.write("CALC:MARK:Y?")                                                   # Query Carrier Level.                 
    powr.append(float(FPL.read()))                                              # Read Value...                        

    FPL.write("CALC:MARK OFF")                                                  # Turn OFF all Markers.                

    FPL.write("DISP:TRAC" + str(n) + ":MODE VIEW")                              # Freeze current Trace.                

    print("")

    if(i % trac_count == 0 and i > 0 and i < pack_count):
        
        n = 0
        
        FPL.write("INST:CRE SAN,'Spectrum " + str(i/trac_count+1) + "'")
        FPL.query("*OPC?")
        
        print("")
        
        FPL.write("INP:ATT:AUTO ON")                                            # Set Input Attenuation.                  
        FPL.write("FREQ:CENT " + str(ul_frq))                                   # Define Center Frequency.                
        FPL.write("FREQ:SPAN " + str(spn))                                      # Define Frequency Span.                  
        FPL.write("SENS:BAND " + str(rbw))                                      # Define Resolution Bandwidth.            
        FPL.write("SENS:BAND:VID " + str(vid))                                  # Define Video Bandwidth.                 
        FPL.write("SENS:SWE:TIME " + str(swe))                                  # Define Duration of Single Sweep.        
        FPL.write("DISP:TRAC:Y:RLEV " + str(rlev))                              # Set Reference Level.                    
        FPL.write("DISP:TRAC:Y:RLEV:OFFS " + str(off))                          # Set Reference Level Offset.             
        FPL.write("TRIG:SOUR IFP")                                              # Select IF Power Trigger Source.         
        FPL.write("TRIG:LEV:IFP " + str(ifp))                                   # Define Power Level for Trigger.         
        
        print("")

print("----------------------------------------------------------------------")

# Identify max. Level and corresponding Freq...

clvl = max(powr)
cfrq = freq[powr.index(clvl)]

# Calculate Definition Points for Limit Line...

x = y = str()

for j in range(0, len(lim)):
    # Horizontal Definition Points:
    x += str(float(cfrq) + xlw[j]) + ',' + str(float(cfrq) + xup[j]) + ','
    # Vertical Definition Points:
    y += str(float(clvl) + lim[j]) + ',' + str(float(clvl) + lim[j]) + ','

## Configuring Limit Line for Sigfox Spectrum                                  
## @ref: R&S FPL Spectrum Analyzer - User Manual                               
##       Literature Number: 1178.3370.02 - 29, September 2017, PDF, p.491ff.   

for i in range(1, int(math.ceil(pack_count/float(trac_count)) + 1)):
    
    print("")

    if(i == 1):
        FPL.write("INST:SEL SAN")
    if(i != 1):
        FPL.write("INST:SEL 'Spectrum " + str(i) + "'")
    
    FPL.write("*OPC?")
    FPL.write("CALC:LIM1:NAME 'SIGFOX_SpectrumLimits'")
    FPL.write("CALC:LIM1:CONT:MODE ABS")
    FPL.write("CALC:LIM1:CONT:SPAC LIN")
    FPL.write("CALC:LIM1:CONT " + x[:-1])
    FPL.write("CALC:LIM1:UPP:MODE ABS")
    FPL.write("CALC:LIM1:UPP "  + y[:-1])
    FPL.write("CALC:LIM1:UPP:SHIF 1.0e-5dB")
    FPL.write("CALC:LIM1:UPP:STAT ON")
    FPL.write("CALC:LIM1:STAT ON")

    rslt += "Frames @ Spectrum " + str(i) + ":"
    
    for j in range(1, pack_count-(i-1)*trac_count+1):

        if j > trac_count: break;

        FPL.write("CALC:LIM:TRAC " + str(j))
        val = FPL.ask("CALC:LIM:FAIL?")

        if val[0] == '0':
            if (j == 1): rslt += '{:>35}'.format("Sigfox_Frame [" + str(j+(i-1)*trac_count) + "] - " +  "PASSED.\n")
            else: rslt += '{:>55}'.format("Sigfox_Frame [" + str(j+(i-1)*trac_count) + "] - " +  "PASSED.\n")
        if val[0] == '1':
            if (j == 1): rslt += '{:>35}'.format("Sigfox_Frame [" + str(j+(i-1)*trac_count) + "] - " +  "FAILED.\n")
            else: rslt += '{:>55}'.format("Sigfox_Frame [" + str(j+(i-1)*trac_count) + "] - " +  "FAILED.\n")

    for k in range(1, pack_count-(i-1)*trac_count+1):
        
        if k > trac_count: break;
        
        FPL.write("CALC:LIM:TRAC" + str(k) + ":CHEC ON")
        
    rslt += "\n"

print("----------------------------------------------------------------------")

plib.reporting.off()
print("----------------------------------------------------------------------")
print("Finishing Module...")
state = ti.read()
print("----------------------------------------------------------------------")
print "State: " + str(stat.replace("\n", ""))
print("----------------------------------------------------------------------")
plib.reporting.on()

# User Information #

if(rslt != ""):
    print("----------------------------------------------------------------------")
    print("Results: Checking Limits in Spectrum.")
    print("----------------------------------------------------------------------")
    print(rslt[:-2])
print("----------------------------------------------------------------------")

# Disconnect Module #

ti.close()
rm.close()

# User Information #

print("----------------------------------------------------------------------")
print("Executing Script finished!")
print("----------------------------------------------------------------------")